`timescale 1ns / 1ps
/*

 - Clk, input
 - 2x Motor, PWM, output
 - 2x contrast sensor, analog input, ADC 10 bit
    - White space, 0  
    - Black space, 1023
 - 3x object detector, digital input, 1 bit
 - Start, Reset Btns, 1 bit input
 - UART Interface
    - Set current motor speed (1-9)
    - Start, Reset
    - Return 0 if success
    - Return 1 if fail
    - UART Protocol,  
        [1:0] bits -> packet type, 0 = Motor set, 1, start, 2 Reset
        [7:2] bits -> packet type == 0, motor speed

*/
/*

    TB hazırlama


*/


module sumoRobot(
    input clk, // 125MHz
    
    output reg [7:0] motor1, // left speed
    output reg [7:0] motor2, // right speed
    
    output reg motor1Direction,
    output reg motor2Direction,
    
    input [9:0] contr1, // left
    input [9:0] contr2, // right
    
    input objDec1, // left
    input objDec2, // mid
    input objDec3, // right
    input startBtn,
    input rst,
    output uartTx,
    input uartRx
);
    
    reg [7:0] uartTxData, uartTxDataNext;
    reg uartTxValid, uartTxValidNext;
    wire uartTxReady;
    
    wire [7:0] uartRxData;
    wire uartRxValid;
    reg uartRxReady, uartRxReadyNext;
    
    uart uart_inst (
        .clk(clk),
        .rst(rst),
        // AXI input
        .s_axis_tdata(uartTxData),
        .s_axis_tvalid(uartTxValid),
        .s_axis_tready(uartTxReady),
        // AXI output
        .m_axis_tdata(uartRxData),
        .m_axis_tvalid(uartRxValid),
        .m_axis_tready(uartRxReady),
        // uart
        .rxd(uartRx),
        .txd(uartTx),
        // status
        .tx_busy(),
        .rx_busy(),
        .rx_overrun_error(),
        .rx_frame_error(),
        // configuration
        .prescale(125000000/(9600*8))
    );
    
    reg [7:0] motor1Next;
    reg [7:0] motor2Next;
    reg [2:0] state, stateNext;
    reg startUart, startUartNext;
    reg resetUart, resetUartNext;
    reg [7:0] motorCurrentSpeed, motorCurrentSpeedNext;
    reg motor1DirectionNext;
    reg motor2DirectionNext;
    reg motorcurrentSpeedInvalid, motorcurrentSpeedInvalidNext;
    reg startUartPacketTx, startUartPacketTxNext;
    reg resetUartPacketTx, resetUartPacketTxNext;
    reg [1:0] uartTxState, uartTxStateNext;
    reg motorSetUartPacketTx, motorSetUartPacketTxNext;
    
    always@(posedge clk) begin
        motor1 <= motor1Next;
        motor2 <= motor2Next;
        uartTxData <= uartTxDataNext;
        uartTxValid <= uartTxValidNext;
        uartRxReady <= uartRxReadyNext;
        state <= stateNext;
        startUart <= startUartNext;
        resetUart <= resetUartNext;
        motorCurrentSpeed <= motorCurrentSpeedNext;
        motor1Direction <= motor1DirectionNext;
        motor2Direction <= motor2DirectionNext;
        motorcurrentSpeedInvalid <= motorcurrentSpeedInvalidNext;
        startUartPacketTx <= startUartPacketTxNext;
        uartTxState <= uartTxStateNext;
        motorSetUartPacketTx <= motorSetUartPacketTxNext;
    end
    
    always@(*) begin
        motor1Next = motor1;
        motor2Next = motor2;
        uartTxDataNext = uartTxData;
        uartTxValidNext = uartTxValid;
        uartRxReadyNext = uartRxReady;
        stateNext = state;
        resetUartNext = resetUart;
        startUartNext = startUart;
        motorCurrentSpeedNext = motorCurrentSpeed;
        uartRxReadyNext = 1;
        motor1DirectionNext = motor1Direction;
        motor2DirectionNext = motor2Direction;
        motorcurrentSpeedInvalidNext = motorcurrentSpeedInvalid;
        startUartPacketTxNext = startUartPacketTx;
        uartTxStateNext = uartTxState;
        motorSetUartPacketTxNext = motorSetUartPacketTx;
        
        if(rst || resetUart) begin
            stateNext = 0;
            uartTxValidNext = 0;
            motor1Next = 0;
            motor2Next = 0;
            resetUartNext = 0;
            motorCurrentSpeedNext = 255;
            startUartPacketTxNext = 0;
            uartTxStateNext = 0;
            motorSetUartPacketTxNext = 0;
        end else begin
        
            case(state)
                0: begin // Wating state
                    if(startBtn || startUart) begin
                        stateNext = 1;
                        startUartNext = 0;
                    end
                end
                1: begin
                    /*output reg motor1, // left
    output reg motor2, // right
    input [9:0] contr1, // left
    input [9:0] contr2, // right
    input objDec1, // left
    input objDec2, // mid
    input objDec3, // right*/
                     if(objDec2) begin
                        motor1Next = motorCurrentSpeed;
                        motor1DirectionNext = 1;
                        motor2Next = motorCurrentSpeed;
                        motor2DirectionNext = 1;
                     end else if(objDec1) begin
                        motor1Next = motorCurrentSpeed;
                        motor1DirectionNext = 0;
                        motor2Next = motorCurrentSpeed;
                        motor2DirectionNext = 1;
                     end else if(objDec3) begin
                        motor1Next = motorCurrentSpeed;
                        motor1DirectionNext = 1;
                        motor2Next = motorCurrentSpeed;
                        motor2DirectionNext = 0;
                     end else begin
                        motor1Next = motorCurrentSpeed >> 1;
                        motor1DirectionNext = 1;
                        motor2Next = motorCurrentSpeed >> 1;
                        motor2DirectionNext = 1;
                     end
                        
                     if(contr1 <100 && contr2 <100) begin
                        motor1Next = motorCurrentSpeed >> 1;
                        motor1DirectionNext = 0;
                        motor2Next = motorCurrentSpeed >> 1;
                        motor2DirectionNext = 0;
                     end else if(contr1 <100) begin
                        motor1Next = motorCurrentSpeed >> 1;
                        motor1DirectionNext = 1;
                        motor2Next = motorCurrentSpeed >> 1;
                        motor2DirectionNext = 0;
                     end else if(contr2 <100) begin
                        motor1Next = motorCurrentSpeed >> 1;
                        motor1DirectionNext = 0;
                        motor2Next = motorCurrentSpeed >> 1;
                        motor2DirectionNext = 1;
                     end
                end
            
            endcase
        end
        
        if(uartRxValid) begin // UART RX
            if(uartRxData[1:0] == 0) begin // Motor Set
               motorSetUartPacketTxNext = 1;
               if(uartRxData[7:2] > 0 && uartRxData[7:2] < 10) begin
                    motorCurrentSpeedNext = uartRxData[7:2];
                    motorcurrentSpeedInvalidNext = 0;
               end else begin
                    motorcurrentSpeedInvalidNext = 1;
               end
            end else if(uartRxData[1:0] == 1) begin //Start
                startUartNext = 1;
                startUartPacketTxNext = 1;
            end else if(uartRxData[1:0] == 2) begin // Reset
                resetUartNext = 1;
                resetUartPacketTxNext = 1;
            end 
        end
        
        case(uartTxState) // Responce
            0: begin
                if(motorSetUartPacketTx) begin
                    uartTxStateNext = 1;
                end else if(startUartPacketTx) begin
                    uartTxStateNext = 2;
                end else if(resetUartPacketTx) begin
                    uartTxStateNext = 2;
                end
            end
            1: begin
                if(uartTxReady) begin
                    uartTxDataNext = motorcurrentSpeedInvalid;
                    uartTxValidNext = 1;
                    uartTxStateNext = 0;
                end
            end
            2: begin
                if(uartTxReady) begin
                    uartTxDataNext = 0; // Success
                    uartTxValidNext = 1;
                    uartTxStateNext = 0;
                end
            end
        endcase
        
        
    end
    
endmodule
